﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace DrawingFunctions
{
    public delegate double DrawnFunction(double x);
    public partial class Form1 : Form
    {
        private List<Function> functions = new List<Function>();

        public Form1()
        {
            InitializeComponent();
            InitializeImage(pictureBox1);
            clearDrawing(pictureBox1);
        }

        private void InitializeImage(PictureBox box)
        {
            box.Image = new Bitmap(box.Width, box.Height);
        }

        private void novyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            InitializeImage(pictureBox1);
            clearDrawing(pictureBox1);
            functions = new List<Function>();
        }

        private void clearDrawing(PictureBox pictureBox1)
        {
            using (Graphics g = Graphics.FromImage(pictureBox1.Image))
            {
                clearBox(g,pictureBox1.Width,pictureBox1.Height);
                drawAxes(g, pictureBox1.Width, pictureBox1.Height);
            }
            pictureBox1.Refresh();
        }

        private static void clearBox(Graphics g,int width,int height)
        {
            g.FillRectangle(Brushes.White, 0, 0, width, height);
        }

        private void drawAxes(Graphics g, int width, int height)
        {
            g.DrawLine(Pens.Black, 0, height / 2, width, height / 2);
            g.DrawLine(Pens.Black, width / 2, 0, width / 2, height);
        }

        private void konecToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void pridatToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddFunction dialog = new AddFunction();
            if(dialog.ShowDialog()==DialogResult.OK)
            {
                functions.Add(dialog.function);
                pictureBox1.Refresh();
            }
        }

        private void pridatToolStripMenuItem_Paint(object sender, PaintEventArgs e)
        {
            clearBox(e.Graphics, pictureBox1.Width, pictureBox1.Height);
            drawAxes(e.Graphics, pictureBox1.Width, pictureBox1.Height);
            foreach(Function func in functions)
            {
                drawFunction(e.Graphics, func, pictureBox1.Width, pictureBox1.Height);
            }
        }

        private void drawFunction(Graphics g, Function func,int width, int height)
        {
            double stepX = 4 * Math.PI / width;
            
            double xVal = -2 * Math.PI;
            int lastY = calculatePoint(func, xVal,height);
            for(int x = 1; x < width; x++)
            {
                xVal += stepX;
                int y = calculatePoint(func, xVal,height);
                Pen pen = new Pen(func.color);
                g.DrawLine(pen, x - 1, lastY, x, y);
                lastY = y;
            }
        }

        private int calculatePoint(Function func, double y,int height)
        {
            double coefY = height / 10.0;
            double yVal = func.a * func.function(func.b * y);
            int pictureY = (int)(height / 2 - (yVal * coefY));
            return pictureY;

        }

        private void ulozitToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Filter = "CSV (*.csv)|*.csv";
            if (saveFileDialog1.ShowDialog()== DialogResult.OK)
            {
                saveData(saveFileDialog1.FileName);
            }
        }

        private void saveData(string fileName)
        {
            using (StreamWriter writer = new StreamWriter(fileName))
            {
                foreach(Function function in functions)
                {
                    writer.WriteLine(function);
                }
            }
        }

        private void nacistToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                loadFunctions(openFileDialog1.FileName);
            }
        }

        private void loadFunctions(string fileName)
        {
            functions.Clear();
            using (StreamReader reader = new StreamReader(fileName))
            {
                string line = reader.ReadLine();
                while (line != null)
                {
                    Function func = line2Function(line);
                    functions.Add(func);
                    line = reader.ReadLine();
                }
            }
            pictureBox1.Refresh();
        }

        private Function line2Function(string line)
        {
            string[] parts = line.Split(';');
            if(parts.Length!= 4)
            {
                throw new InvalidDataException("Wrong line format: " + line);
            }
            Function func = new Function();
            int funcId = Int32.Parse(parts[0]);
            if(funcId == 0)
            {
                func.function = Math.Sin;
            }
            else
            {
                func.function = Math.Cos;
            }
            func.a = Int16.Parse(parts[1]);
            func.b = Int16.Parse(parts[2]);
            func.color = Color.FromName(parts[3]);
            return func;
        }

        private void exportDoJPGToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Filter = "JPEG (*.jpg)|*.jpg";
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                using(Graphics g = Graphics.FromImage(pictureBox1.Image))
                {
                    foreach(Function func in functions)
                    {
                        drawFunction(g, func, pictureBox1.Width, pictureBox1.Height);
                    }
                }
                pictureBox1.Image.Save(saveFileDialog1.FileName, ImageFormat.Jpeg);
            }
        }
    }

    public class Function
    {
        public DrawnFunction function { set; get; }
        public int a { get; set; }
        public int b { get; set; }
        public Color color { get; set; }

        public override string ToString()
        {
            string value = "";
            if(function == Math.Sin)
            {
                value += "0;";
            }
            else
            {
                value += "1;";
            }
            value += a + ";" + b + ";" + color.Name;
            return value;

        }
    }
}
